@@ -70,15 +70,15 @@ where
 
         let names = read_names(&mut self.inner)?;
         let references = read_references(&mut self.inner, n_ref)?;
-        let n_no_coors = self.inner.read_u64::<LittleEndian>().ok();
+        let n_no_coor = read_unplaced_unmapped_record_count(&mut self.inner)?;
 
         let mut builder = Index::builder()
             .set_header(header)
             .set_reference_sequence_names(names)
             .set_reference_sequences(references);
 
-        if let Some(unmapped_read_count) = n_no_coors {
-            builder = builder.set_unmapped_read_count(unmapped_read_count);
+        if let Some(unplaced_unmapped_record_count) = n_no_coor {
+            builder = builder.set_unplaced_unmapped_record_count(unplaced_unmapped_record_count);
         }
 
         Ok(builder.build())
@@ -171,10 +171,14 @@ fn parse_names(buf: &[u8]) -> io::Result<ReferenceSequenceNames> {
             Some(end) => {
                 let raw_name = &buf[..end];
                 let name = str::from_utf8(raw_name)
-                    .map(|s| s.into())
                     .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
 
-                names.insert(name);
+                if !names.insert(name.into()) {
+                    return Err(io::Error::new(
+                        io::ErrorKind::InvalidData,
+                        format!("duplicate reference sequence name: {}", name),
+                    ));
+                }
 
                 start += end + 1;
             }
@@ -306,6 +310,17 @@ where
     Ok(Metadata::new(ref_beg, ref_end, n_mapped, n_unmapped))
 }
 
+fn read_unplaced_unmapped_record_count<R>(reader: &mut R) -> io::Result<Option<u64>>
+where
+    R: Read,
+{
+    match reader.read_u64::<LittleEndian>() {
+        Ok(n) => Ok(Some(n)),
+        Err(ref e) if e.kind() == io::ErrorKind::UnexpectedEof => Ok(None),
+        Err(e) => Err(e),
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -429,6 +444,16 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_parse_names_with_duplicate_name() {
+        let data = b"sq0\x00sq0\x00";
+
+        assert!(matches!(
+            parse_names(data),
+            Err(ref e) if e.kind() == io::ErrorKind::InvalidData,
+        ));
+    }
+
     #[test]
     fn test_read_metadata() -> io::Result<()> {
         let data = [
@@ -453,4 +478,17 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn test_read_unplaced_unmapped_record_count() -> io::Result<()> {
+        let data = [];
+        let mut reader = &data[..];
+        assert_eq!(read_unplaced_unmapped_record_count(&mut reader)?, None);
+
+        let data = [0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00];
+        let mut reader = &data[..];
+        assert_eq!(read_unplaced_unmapped_record_count(&mut reader)?, Some(8));
+
+        Ok(())
+    }
 }
